@@ -1,4 +1,4 @@
-# Copyright 2015 Google Inc. All Rights Reserved.
+# Copyright 2015-2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -48,12 +48,13 @@ def Reformat(uwlines, verify=True):
   """
   final_lines = []
   prev_uwline = None  # The previous line.
+  indent_width = style.Get('INDENT_WIDTH')
 
   for uwline in _SingleOrMergedLines(uwlines):
     first_token = uwline.first
-    _FormatFirstToken(first_token, uwline.depth, prev_uwline)
+    _FormatFirstToken(first_token, uwline.depth, prev_uwline, final_lines)
 
-    indent_amt = style.Get('INDENT_WIDTH') * uwline.depth
+    indent_amt = indent_width * uwline.depth
     state = format_decision_state.FormatDecisionState(uwline, indent_amt)
 
     if not uwline.disable:
@@ -73,12 +74,13 @@ def Reformat(uwlines, verify=True):
       _RetainHorizontalSpacing(uwline)
       _RetainVerticalSpacing(uwline, prev_uwline)
       _EmitLineUnformatted(state)
-    elif _CanPlaceOnSingleLine(uwline):
+    elif (_CanPlaceOnSingleLine(uwline) and not any(tok.must_split
+                                                    for tok in uwline.tokens)):
       # The unwrapped line fits on one line.
       while state.next_token:
         state.AddTokenToState(newline=False, dry_run=False)
     else:
-      if not _AnalyzeSolutionSpace(state, dry_run=False):
+      if not _AnalyzeSolutionSpace(state):
         # Failsafe mode. If there isn't a solution to the line, then just emit
         # it as is.
         state = format_decision_state.FormatDecisionState(uwline, indent_amt)
@@ -88,26 +90,7 @@ def Reformat(uwlines, verify=True):
 
     final_lines.append(uwline)
     prev_uwline = uwline
-
-  formatted_code = []
-  for line in final_lines:
-    formatted_line = []
-    for tok in line.tokens:
-      if not tok.is_pseudo_paren:
-        formatted_line.append(tok.whitespace_prefix)
-        formatted_line.append(tok.value)
-      else:
-        if (not tok.next_token.whitespace_prefix.startswith('\n') and
-            not tok.next_token.whitespace_prefix.startswith(' ')):
-          if (tok.previous_token.value == ':' or
-              tok.next_token.value not in ',}])'):
-            formatted_line.append(' ')
-
-    formatted_code.append(''.join(formatted_line))
-    if verify:
-      verifier.VerifyCode(formatted_code[-1])
-
-  return ''.join(formatted_code) + '\n'
+  return _FormatFinalLines(final_lines, verify)
 
 
 def _RetainHorizontalSpacing(uwline):
@@ -132,6 +115,11 @@ def _RetainVerticalSpacingBetweenTokens(cur_tok, prev_tok):
 
   if prev_tok.is_string:
     prev_lineno = prev_tok.lineno + prev_tok.value.count('\n')
+  elif prev_tok.is_pseudo_paren:
+    if not prev_tok.previous_token.is_multiline_string:
+      prev_lineno = prev_tok.previous_token.lineno
+    else:
+      prev_lineno = prev_tok.lineno
   else:
     prev_lineno = prev_tok.lineno
 
@@ -233,6 +221,28 @@ def _CanPlaceOnSingleLine(uwline):
           not any(tok.is_comment for tok in uwline.tokens[:-1]))
 
 
+def _FormatFinalLines(final_lines, verify):
+  formatted_code = []
+  for line in final_lines:
+    formatted_line = []
+    for tok in line.tokens:
+      if not tok.is_pseudo_paren:
+        formatted_line.append(tok.whitespace_prefix)
+        formatted_line.append(tok.value)
+      else:
+        if (not tok.next_token.whitespace_prefix.startswith('\n') and
+            not tok.next_token.whitespace_prefix.startswith(' ')):
+          if (tok.previous_token.value == ':' or
+              tok.next_token.value not in ',}])'):
+            formatted_line.append(' ')
+
+    formatted_code.append(''.join(formatted_line))
+    if verify:
+      verifier.VerifyCode(formatted_code[-1])
+
+  return ''.join(formatted_code) + '\n'
+
+
 class _StateNode(object):
   """An edge in the solution space from 'previous.state' to 'state'.
 
@@ -251,7 +261,7 @@ class _StateNode(object):
     self.newline = newline
     self.previous = previous
 
-  def __repr__(self):
+  def __repr__(self):  # pragma: no cover
     return 'StateNode(state=[\n{0}\n], newline={1})'.format(self.state,
                                                             self.newline)
 
@@ -267,7 +277,7 @@ _QueueItem = collections.namedtuple('QueueItem', ['ordered_penalty',
                                                   'state_node'])
 
 
-def _AnalyzeSolutionSpace(initial_state, dry_run=False):
+def _AnalyzeSolutionSpace(initial_state):
   """Analyze the entire solution space starting from initial_state.
 
   This implements a variant of Dijkstra's algorithm on the graph that spans
@@ -278,7 +288,6 @@ def _AnalyzeSolutionSpace(initial_state, dry_run=False):
   Arguments:
     initial_state: (format_decision_state.FormatDecisionState) The initial state
       to start the search from.
-    dry_run: (bool) Don't commit changes if True.
 
   Returns:
     True if a formatting solution was found. False otherwise.
@@ -307,9 +316,7 @@ def _AnalyzeSolutionSpace(initial_state, dry_run=False):
     if node.state in seen:
       continue
 
-    assert penalty >= prev_penalty
     prev_penalty = penalty
-
     seen.add(node.state)
 
     # FIXME(morbo): Add a 'decision' element?
@@ -321,9 +328,7 @@ def _AnalyzeSolutionSpace(initial_state, dry_run=False):
     # We weren't able to find a solution. Do nothing.
     return False
 
-  if not dry_run:
-    _ReconstructPath(initial_state, heapq.heappop(p_queue).state_node)
-
+  _ReconstructPath(initial_state, heapq.heappop(p_queue).state_node)
   return True
 
 
@@ -347,12 +352,14 @@ def _AddNextStateToQueue(penalty, previous_node, newline, count, p_queue):
   if newline and not previous_node.state.CanSplit():
     # Don't add a newline if the token cannot be split.
     return count
-  if not newline and previous_node.state.MustSplit():
+  must_split = previous_node.state.MustSplit()
+  if not newline and must_split:
     # Don't add a token we must split but where we aren't splitting.
     return count
 
   node = _StateNode(previous_node.state, newline, previous_node)
-  penalty += node.state.AddTokenToState(newline=newline, dry_run=True)
+  penalty += node.state.AddTokenToState(
+      newline=newline, dry_run=True, must_split=must_split)
   heapq.heappush(p_queue, _QueueItem(_OrderedPenalty(penalty, count), node))
   return count + 1
 
@@ -376,7 +383,7 @@ def _ReconstructPath(initial_state, current):
     initial_state.AddTokenToState(newline=node.newline, dry_run=False)
 
 
-def _FormatFirstToken(first_token, indent_depth, prev_uwline):
+def _FormatFirstToken(first_token, indent_depth, prev_uwline, final_lines):
   """Format the first token in the unwrapped line.
 
   Add a newline and the required indent before the first token of the unwrapped
@@ -388,9 +395,12 @@ def _FormatFirstToken(first_token, indent_depth, prev_uwline):
     indent_depth: (int) The line's indentation depth.
     prev_uwline: (list of unwrapped_line.UnwrappedLine) The unwrapped line
       previous to this line.
+    final_lines: (list of unwrapped_line.UnwrappedLine) The unwrapped lines
+      that have already been processed.
   """
   first_token.AddWhitespacePrefix(
-      _CalculateNumberOfNewlines(first_token, indent_depth, prev_uwline),
+      _CalculateNumberOfNewlines(first_token, indent_depth, prev_uwline,
+                                 final_lines),
       indent_level=indent_depth)
 
 
@@ -399,7 +409,8 @@ ONE_BLANK_LINE = 2
 TWO_BLANK_LINES = 3
 
 
-def _CalculateNumberOfNewlines(first_token, indent_depth, prev_uwline):
+def _CalculateNumberOfNewlines(first_token, indent_depth, prev_uwline,
+                               final_lines):
   """Calculate the number of newlines we need to add.
 
   Arguments:
@@ -408,6 +419,8 @@ def _CalculateNumberOfNewlines(first_token, indent_depth, prev_uwline):
     indent_depth: (int) The line's indentation depth.
     prev_uwline: (list of unwrapped_line.UnwrappedLine) The unwrapped line
       previous to this line.
+    final_lines: (list of unwrapped_line.UnwrappedLine) The unwrapped lines
+      that have already been processed.
 
   Returns:
     The number of newlines needed before the first token.
@@ -418,7 +431,7 @@ def _CalculateNumberOfNewlines(first_token, indent_depth, prev_uwline):
     # The first line in the file. Don't add blank lines.
     # FIXME(morbo): Is this correct?
     if first_token.newlines is not None:
-      pytree_utils.SetNodeAnnotation(first_token.GetPytreeNode(),
+      pytree_utils.SetNodeAnnotation(first_token.node,
                                      pytree_utils.Annotation.NEWLINES, None)
     return 0
 
@@ -429,7 +442,7 @@ def _CalculateNumberOfNewlines(first_token, indent_depth, prev_uwline):
 
   prev_last_token = prev_uwline.last
   if prev_last_token.is_docstring:
-    if not indent_depth and first_token.value in {'class', 'def'}:
+    if (not indent_depth and first_token.value in {'class', 'def', 'async'}):
       # Separate a class or function from the module-level docstring with two
       # blank lines.
       return TWO_BLANK_LINES
@@ -451,9 +464,17 @@ def _CalculateNumberOfNewlines(first_token, indent_depth, prev_uwline):
                                            prev_last_token):
           # Assume that the comment is "attached" to the current line.
           # Therefore, we want two blank lines before the comment.
-          prev_last_token.AdjustNewlinesBefore(TWO_BLANK_LINES)
+          index = len(final_lines) - 1
+          while index > 0:
+            if not final_lines[index - 1].is_comment:
+              break
+            index -= 1
+          if final_lines[index - 1].first.value == '@':
+            final_lines[index].first.AdjustNewlinesBefore(NO_BLANK_LINES)
+          else:
+            prev_last_token.AdjustNewlinesBefore(TWO_BLANK_LINES)
           if first_token.newlines is not None:
-            pytree_utils.SetNodeAnnotation(first_token.GetPytreeNode(),
+            pytree_utils.SetNodeAnnotation(first_token.node,
                                            pytree_utils.Annotation.NEWLINES,
                                            None)
           return NO_BLANK_LINES
@@ -470,7 +491,11 @@ def _CalculateNumberOfNewlines(first_token, indent_depth, prev_uwline):
   else:
     first_token_lineno = first_token.lineno
 
-  if first_token_lineno - prev_last_token.lineno > 1:
+  prev_last_token_lineno = prev_last_token.lineno
+  if prev_last_token.is_multiline_string:
+    prev_last_token_lineno += prev_last_token.value.count('\n')
+
+  if first_token_lineno - prev_last_token_lineno > 1:
     return ONE_BLANK_LINE
 
   return NO_BLANK_LINES
@@ -497,9 +522,8 @@ def _SingleOrMergedLines(uwlines):
         if uwlines[index].lineno != uwline.lineno:
           break
         if uwline.last.value != ':':
-          leaf = pytree.Leaf(type=token.SEMI,
-                             value=';',
-                             context=('', (uwline.lineno, column)))
+          leaf = pytree.Leaf(
+              type=token.SEMI, value=';', context=('', (uwline.lineno, column)))
           uwline.AppendToken(format_token.FormatToken(leaf))
         for tok in uwlines[index].tokens:
           uwline.AppendToken(tok)
