@@ -1,4 +1,4 @@
-# Copyright 2015 Google Inc. All Rights Reserved.
+# Copyright 2015-2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,6 +24,7 @@ from yapf.yapflib import pytree_visitor
 from yapf.yapflib import split_penalty
 
 UNBREAKABLE = split_penalty.UNBREAKABLE
+DOTTED_NAME = split_penalty.DOTTED_NAME
 STRONGLY_CONNECTED = split_penalty.STRONGLY_CONNECTED
 CONTIGUOUS_LIST = split_penalty.CONTIGUOUS_LIST
 
@@ -163,11 +164,11 @@ class SplitPenaltyTest(unittest.TestCase):
     tree = self._ParseAndComputePenalties(code)
     self._CheckPenalties(tree, [
         ('a', None), ('=', None), ('{', None),
-        ("'x'", STRONGLY_CONNECTED),
+        ("'x'", None),
         (':', STRONGLY_CONNECTED),
         ('42', None),
         (',', None),
-        ('y', STRONGLY_CONNECTED),
+        ('y', None),
         ('(', UNBREAKABLE),
         ('lambda', STRONGLY_CONNECTED),
         ('a', UNBREAKABLE),
@@ -187,18 +188,18 @@ class SplitPenaltyTest(unittest.TestCase):
     tree = self._ParseAndComputePenalties(code)
     self._CheckPenalties(tree, [
         ('[', None),
-        ('a', CONTIGUOUS_LIST),
-        ('for', 0),
+        ('a', STRONGLY_CONNECTED),
+        ('for', STRONGLY_CONNECTED),
         ('a', STRONGLY_CONNECTED),
         ('in', STRONGLY_CONNECTED),
         ('foo', STRONGLY_CONNECTED),
-        ('if', 0),
+        ('if', STRONGLY_CONNECTED),
         ('a', STRONGLY_CONNECTED),
         ('.', UNBREAKABLE),
-        ('x', UNBREAKABLE),
+        ('x', DOTTED_NAME),
         ('==', STRONGLY_CONNECTED),
         ('37', STRONGLY_CONNECTED),
-        (']', CONTIGUOUS_LIST),
+        (']', STRONGLY_CONNECTED),
     ])  # yapf: disable
 
   def testFuncCalls(self):
